-- Filename : wgenerator_SN54xx27.vhd
--
-- Version 2.1
----------------------------------------------------------------------
--
  use STD.TEXTIO.all;

library WAVES_STANDARD;
  use WAVES_STANDARD.WAVES_SYSTEM.all;
  use WAVES_STANDARD.WAVES_INTERFACE.all;

library SN54xx27;
  use SN54xx27.WAVES_DEVICE.all;
  use SN54xx27.WAVES_OBJECTS.all;
  use SN54xx27.WAVES_FRAMES.all;
  use SN54xx27.WAVES_UTILITIES.all;

package WGENERATOR_SN54xx27 is
  procedure SN54xx27_TEST
  (
    signal CONNECT   : inout Waves_Port_List
  );

  procedure SN54xx27_TEST
  (
    signal CONNECT            : inout Waves_Port_List;
    constant INPUT_FILE_NAME  : String;
    constant START_TOLERANCE  : Time;
    constant END_TOLERANCE    : Time 
  );

end WGENERATOR_SN54xx27;

-- ======================================================================

package body WGENERATOR_SN54xx27 is

    constant O_PINS : Pinset := NEW_PINSET(( TP_Y1, TP_Y2, TP_Y3));
    constant I_PINS : Pinset := ALL_PINS and not O_PINS;
    constant EMPTY_EVENT : Integer := -1;

  ---------------------------------------------------------------------------
  --  The following procedure is used to provide test vectors for the model.
  --  The test vectors (inputs and expected outputs) are contained within
  --  the procedure, and are analyzed with the VHDL code.  The test vectors
  --  are applied as follows:
  --
  --   Beginning of period:  Inputs set to appropriate values
  --                         Expected outputs set to "don't care"
  --
  --   After 95 ns (95% of period):  Expected outputs set to appropriate values
  --
  --   After 100 ns (end of period): Get next set of inputs and outputs
  ---------------------------------------------------------------------------
  procedure SN54xx27_TEST(
    signal CONNECT   : inout Waves_Port_List) is

    constant PERIOD          : Time       := 100 ns;
    constant UNCERTAINTY_PCT : Real       := 0.95;
    constant TDELAY          : Event_Time := ETIME (UNCERTAINTY_PCT * PERIOD);

    constant FSA : Frame_Set_Array := 
      NEW_FRAME_SET_ARRAY ( INPUT_FRAMES, I_PINS) +
      NEW_FRAME_SET_ARRAY ( STROBED_OUTPUT_FRAMES(TDELAY), O_PINS);

    constant DT_BASIS : Delay_Time_Basis := (FALSE, 0 ns, 0 ns, 
                                             TIMED_DELAY, 0, 0, 0);

    variable TD1 : Time_Data := NEW_TIME_DATA ( FSA );
    variable T1  : Wave_Timing := ((PERIOD, DT_BASIS), TD1);

    --        Order of signals:
    --
    --        (Y1, Y2, Y3,
    --         A1, B1, C1, A2, B2, C2, A3, B3, C3)
    --

    variable ALL_STATES_PART1 : Wtime_Slice_List (1 to 11) := (
      (  "UUU"
     & "UUUUUUUUU", T1),

      (  "111"
     & "000000000", T1), -- reset to normal

      (  "UUU"
     & "00UUUUUUU", T1), -- 9 states 

      (  "XUU"
     & "00XUUUUUU", T1),

      (  "1UU"
     & "000UUUUUU", T1),

      (  "0UU"
     & "001UUUUUU", T1),

      (  "XUU"
     & "00ZUUUUUU", T1),

      (  "XUU"
     & "00WUUUUUU", T1),

      (  "1UU"
     & "00LUUUUUU", T1),

      (  "0UU"
     & "00HUUUUUU", T1),

      (  "XUU"
     & "00-UUUUUU", T1)
      );



    variable ALL_STATES_PART2 : Wtime_Slice_List (1 to 9) := (
      (  "UUU"
     & "UUU00UUUU", T1), -- 9 states

      (  "UXU"
     & "UUU00XUUU", T1), 

      (  "U1U"
     & "UUU000UUU", T1),

      (  "U0U"
     & "UUU001UUU", T1),

      (  "UXU"
     & "UUU00ZUUU", T1),

      (  "UXU"
     & "UUU00WUUU", T1),

      (  "U1U"
     & "UUU00LUUU", T1),

      (  "U0U"
     & "UUU00HUUU", T1),

      (  "UXU"
     & "UUU00-UUU", T1)
      );



    variable ALL_STATES_PART3 : Wtime_Slice_List (1 to 9) := (
      (  "UUU"
     & "UUUUUU00U", T1), -- 9 states 

      (  "UUX"
     & "UUUUUU00X", T1),

      (  "UU1"
     & "UUUUUU000", T1),

      (  "UU0"
     & "UUUUUU001", T1),

      (  "UUX"
     & "UUUUUU00Z", T1),

      (  "UUX"
     & "UUUUUU00W", T1),

      (  "UU1"
     & "UUUUUU00L", T1),

      (  "UU0"
     & "UUUUUU00H", T1),

      (  "UU-"
     & "UUUUUU00-", T1)
      );



    variable TIMING_PARAMS : Wtime_Slice_List (1 to 4) := (
      (  "111"
     & "000000000", T1), -- reset to normal

      (  "111"
     & "000000000", T1), -- timing parameters

      (  "000"
     & "111111111", T1), -- tphl

      (  "111"
     & "000000000", T1) -- tplh
      );



    variable TRUTH_TABLE : Wtime_Slice_List (1 to 5) := (
      (  "111"
     & "000000000", T1), -- reset to normal

      (  "111"
     & "000000000", T1), -- truth table

      (  "000"
     & "1XX1XX1XX", T1),

      (  "000"
     & "X1XX1XX1X", T1),

      (  "000"
     & "XX1XX1XX1", T1)
      );



  begin
    assert FALSE
      report "ALL_STATES_PART1 vectors"
        severity note;
      for I in ALL_STATES_PART1' range loop
        APPLY(CONNECT,ALL_STATES_PART1(I).CODES,ALL_STATES_PART1(I).WTIME,ALL_PINS);
      end loop;

    assert FALSE
      report "ALL_STATES_PART2 vectors"
        severity note;
      for I in ALL_STATES_PART2' range loop
        APPLY(CONNECT,ALL_STATES_PART2(I).CODES,ALL_STATES_PART2(I).WTIME,ALL_PINS);
      end loop;

    assert FALSE
      report "ALL_STATES_PART3 vectors"
        severity note;
      for I in ALL_STATES_PART3' range loop
        APPLY(CONNECT,ALL_STATES_PART3(I).CODES,ALL_STATES_PART3(I).WTIME,ALL_PINS);
      end loop;

    assert FALSE
      report "TIMING_PARAMS vectors"
        severity note;
      for I in TIMING_PARAMS' range loop
        APPLY(CONNECT,TIMING_PARAMS(I).CODES,TIMING_PARAMS(I).WTIME,ALL_PINS);
      end loop;

    assert FALSE
      report "TRUTH_TABLE vectors"
        severity note;
      for I in TRUTH_TABLE' range loop
        APPLY(CONNECT,TRUTH_TABLE(I).CODES,TRUTH_TABLE(I).WTIME,ALL_PINS);
      end loop;

    end;

  ---------------------------------------------------------------------------
  --  The following procedure is used to provide test vectors for the model.
  --  The test vectors (inputs and expected outputs, along with the period
  --  of each slice) are read in from an external file.  The test vectors 
  --  are applied as follows:
  --
  --   Beginning of period:  Inputs set to appropriate values
  --                         Expected outputs set to "don't care"
  --
  --   After <START_TOLERANCE>:  Expected outputs set to appropriate values
  --
  --   After <FS_TIME - END_TOLERANCE>: Get next set of inputs and outputs
  --
  --   After <FS_TIME> (end of period): Get next set of inputs and outputs
  ---------------------------------------------------------------------------
    procedure SN54xx27_Test
    (
      signal CONNECT            : inout Waves_Port_List;
      constant INPUT_FILE_NAME  : String;
      constant START_TOLERANCE  : Time;
      constant END_TOLERANCE    : Time ) is

      variable TOLERANCE : Time := START_TOLERANCE + END_TOLERANCE;

      constant FSA : Frame_Set_Array :=
        NEW_FRAME_SET_ARRAY (INPUT_FRAMES, I_PINS) +
        NEW_FRAME_SET_ARRAY (OUTPUT_FRAMES, O_PINS);

      constant DT_BASIS : Delay_Time_Basis := (FALSE, 0 ns, 0 ns, TIMED_DELAY, 0, 0, 0);

      variable TD : Time_Data := NEW_TIME_DATA (FSA);

      constant FSA_DONT_CARE : Frame_Set_Array :=
        NEW_FRAME_SET_ARRAY (INPUT_FRAMES, I_PINS) +
        NEW_FRAME_SET_ARRAY (DONT_CARE_FRAMES, O_PINS);

      variable TD_DONT_CARE : Time_Data := NEW_TIME_DATA (FSA_DONT_CARE);

      file INPUT_FILE : Text is in INPUT_FILE_NAME;

      -- the file slice must be allocated
      variable SN54xx27_FILE_SLICE : File_Slice := NEW_FILE_SLICE;

    begin
      loop
        READ_FILE_SLICE(INPUT_FILE, SN54xx27_FILE_SLICE);
        exit when SN54xx27_FILE_SLICE.END_OF_FILE;
        if SN54xx27_FILE_SLICE.FS_TIME > TOLERANCE then
          APPLY(CONNECT, SN54xx27_FILE_SLICE.CODES.all, 
                DELAY(START_TOLERANCE), TD_DONT_CARE);
          APPLY(CONNECT, SN54xx27_FILE_SLICE.CODES.all, 
                DELAY(SN54xx27_FILE_SLICE.FS_TIME - TOLERANCE), TD);
          APPLY(CONNECT, SN54xx27_FILE_SLICE.CODES.all, 
                DELAY(END_TOLERANCE), TD_DONT_CARE);
        else
          APPLY(CONNECT, SN54xx27_FILE_SLICE.CODES.all, 
                DELAY(SN54xx27_FILE_SLICE.FS_TIME), TD);
        end if;
      end loop;
    end;
end WGENERATOR_SN54xx27;
